package org.postgresql.util;

import java.io.*;
import java.sql.*;

/*
 * This class extends Exception
 */
public class FailOverException
    extends Exception {
  private String message;

  /*
   * This provides the same functionality to Exception
   * @param error Error string
   */
  public FailOverException(String error) {
    super();
    message = error;
  }

  /*
   * Overides Throwable
   */
  public String getMessage() {
    return message;
  }

  /*
   * Overides Object
   */
  public String toString() {
    return message;
  }
}