package org.postgresql.util;

import java.sql.SQLException;
import java.net.MulticastSocket;
import java.net.InetAddress;
import java.net.DatagramPacket;


public class Replica {

  private String ip;
  private int port;

  public Replica (String ip, int port) {
    this.ip = ip;
    this.port = port;
  }

  public void setIp (String ip) {
    this.ip = ip;
  }

  public void setPort (int port) {
    this.port = port;
  }

  public String getIp () {
    return this.ip;
  }

  public int getPort () {
    return this.port;
  }
  
  public String toString () {
    return ip + ":" + port;
  }
};

