/*
 * Created on 02-nov-2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.postgresql.coreDriver;

/**
 * @author jsalas
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class ContextTaskReceiveFields {
	int size;
	int round;
	
	String typeName = null;
	int typeOid = 0;
	int typeLength = 0;
	int typeModifier = 0;
	
	/**
	 * 
	 */
	public ContextTaskReceiveFields() {
		this.size = 0;
		this.round = 0;
	}
		
	/**
	 * @param size
	 * @param round
	 */
	public ContextTaskReceiveFields(int size, int round) {
		this.size = size;
		this.round = round;
	}
	
	/* ---------- Getters and Setters ---------- */
	/**
	 * @return Returns the round.
	 */
	public int getRound() {
		return round;
	}
	/**
	 * @param round The round to set.
	 */
	public void setRound(int round) {
		this.round = round;
	}
	/**
	 * @return Returns the size.
	 */
	public int getSize() {
		return size;
	}
	/**
	 * @param size The size to set.
	 */
	public void setSize(int size) {
		this.size = size;
	}
	
	
	/**
	 * @return Returns the typeLength.
	 */
	public int getTypeLength() {
		return typeLength;
	}
	/**
	 * @param typeLength The typeLength to set.
	 */
	public void setTypeLength(int typeLength) {
		this.typeLength = typeLength;
	}
	/**
	 * @return Returns the typeModifier.
	 */
	public int getTypeModifier() {
		return typeModifier;
	}
	/**
	 * @param typeModifier The typeModifier to set.
	 */
	public void setTypeModifier(int typeModifier) {
		this.typeModifier = typeModifier;
	}
	/**
	 * @return Returns the typeName.
	 */
	public String getTypeName() {
		return typeName;
	}
	/**
	 * @param typeName The typeName to set.
	 */
	public void setTypeName(String typeName) {
		this.typeName = typeName;
	}
	/**
	 * @return Returns the typeOid.
	 */
	public int getTypeOid() {
		return typeOid;
	}
	/**
	 * @param typeOid The typeOid to set.
	 */
	public void setTypeOid(int typeOid) {
		this.typeOid = typeOid;
	}
}
